#pragma once

#include <SFML/Graphics/RectangleShape.hpp>
#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Graphics/Transform.hpp>
#include <SFML/Graphics/Transformable.hpp>
#include <SFML/System/Vector2.hpp>
#include <SFML/Window/Event.hpp>

# include "ILocationProvider.hpp"

inline sf::Transformable transformFromPosRot(sf::Vector2f position, float rotation) {
    auto result = sf::Transformable();
    result.setPosition(position);
    result.setRotation(rotation);
    return result;
}

class PosTracker : public ILocationProvider {
    /// Wheel encoder values.
    float _wheelDistance, _wheelDiameter;
    float _wheelStepsPerTurn;
    float _lastLeftValue, _lastRightValue;
    float _wheel_dir_left, _wheel_dir_right;
    sf::Transformable transform;
    public:
      PosTracker(sf::Vector2f initialPos, float initialRot, float wheelDistance,
                 float wheelDiameter, float wheelStepsPerTurn)
          : _wheelDistance(wheelDistance), _wheelDiameter(wheelDiameter),
            _wheelStepsPerTurn(wheelStepsPerTurn), _lastLeftValue(0.),
            _lastRightValue(0.),
            _wheel_dir_left(1.),
            _wheel_dir_right(1.),
            transform(transformFromPosRot(initialPos, initialRot))
        {}
        void reset(sf::Vector2f initialPos, float initialRot) override;
        void update(std::size_t value_left, std::size_t value_right) override;
        virtual void hint_right(bool forward) override;
        virtual void hint_left(bool forward) override;

        float get_rot() override;
        sf::Vector2f get_pos() override;
    private:
        inline float wheelIncrementDistance() const {
            return _wheelDiameter * std::numbers::pi / _wheelStepsPerTurn;
        }
        void updateTransform(float leftDistance, float rightDistance);
};
